/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.api.type.component;

import nerdhub.cardinal.components.api.ComponentType;
import nerdhub.cardinal.components.api.component.Component;
import nerdhub.cardinal.components.api.component.extension.CopyableComponent;
import net.minecraft.class_1160;
import net.minecraft.class_1309;
import net.minecraft.class_2487;
import net.minecraft.class_310;
import net.minecraft.class_4587;
import net.minecraft.class_4597;
import net.minecraft.class_572;
import net.minecraft.class_583;
import net.minecraft.class_630;
import net.minecraft.class_897;
import net.minecraft.class_922;
import top.theillusivec4.curios.api.CuriosComponent;
import top.theillusivec4.curios.api.type.ISlotType;

public interface IRenderableCurio extends CopyableComponent<IRenderableCurio> {

  /**
   * Performs rendering of the ItemStack. Note that vertical sneaking translations are automatically
   * applied before this rendering method is called.
   *
   * @param identifier   The identifier of the {@link ISlotType} of the slot
   * @param livingEntity The LivingEntity that is wearing the ItemStack
   */
  default void render(String identifier, int index, class_4587 matrixStack,
      class_4597 vertexConsumerProvider, int light, class_1309 livingEntity,
      float limbSwing, float limbSwingAmount, float partialTicks, float ageInTicks,
      float netHeadYaw, float headPitch) {

  }

  @Override
  default void fromTag(class_2487 var1) {

  }

  @Override
  default class_2487 toTag(class_2487 var1) {
    return new class_2487();
  }

  @Override
  default boolean isComponentEqual(Component other) {
    return true;
  }

  @Override
  default ComponentType<IRenderableCurio> getComponentType() {
    return CuriosComponent.ITEM_RENDER;
  }

  /**
   * Some helper methods for rendering curios.
   */
  final class RenderHelper {

    /**
     * Translates the rendering for the curio if the entity is sneaking.
     *
     * @param livingEntity The wearer of the curio
     */
    public static void translateIfSneaking(final class_4587 matrixStack,
        final class_1309 livingEntity) {

      if (livingEntity.method_5715()) {
        matrixStack.method_22904(0.0f, 0.2f, 0.0f);
      }
    }

    /**
     * Rotates the rendering for the curio if the entity is sneaking. The rotation angle is based on
     * the body of a player model when sneaking, so this is typically used for items being rendered
     * on the body.
     *
     * @param livingEntity The wearer of the curio
     */
    public static void rotateIfSneaking(final class_4587 matrixStack,
        final class_1309 livingEntity) {

      if (livingEntity.method_5715()) {
        matrixStack.method_22907(class_1160.field_20703.method_23214(90.0F / (float) Math.PI));
      }
    }

    /**
     * Rotates the rendering for the model renderers based on the entity's head movement. This will
     * align the model renderers with the movements and rotations of the head. This will do nothing
     * if the entity render object does not implement {@link LivingEntityRenderer} or if the model
     * does not have a head (does not implement {@link BipedEntityModel}).
     *
     * @param livingEntity The wearer of the curio
     * @param parts        The list of model renderers to align to the head movement
     */
    public static void followHeadRotations(final class_1309 livingEntity, class_630... parts) {

      class_897<? super class_1309> render = class_310.method_1551()
          .method_1561().method_3953(livingEntity);

      if (render instanceof class_922) {
        @SuppressWarnings("unchecked") class_922<class_1309, class_583<class_1309>> livingRenderer = (class_922<class_1309, class_583<class_1309>>) render;
        class_583<class_1309> model = livingRenderer.method_4038();

        if (model instanceof class_572) {

          for (class_630 part : parts) {
            part.method_17138(((class_572<class_1309>) model).field_3398);
          }
        }
      }
    }

    /**
     * Rotates the rendering for the models based on the entity's poses and movements. This will do
     * nothing if the entity render object does not implement {@link LivingEntityRenderer} or if the
     * model does not implement {@link BipedEntityModel}).
     *
     * @param livingEntity The wearer of the curio
     * @param models       The list of models to align to the body movement
     */
    @SafeVarargs
    public static void followBodyRotations(final class_1309 livingEntity,
        final class_572<class_1309>... models) {

      class_897<? super class_1309> render = class_310.method_1551()
          .method_1561().method_3953(livingEntity);

      if (render instanceof class_922) {
        @SuppressWarnings("unchecked") class_922<class_1309, class_583<class_1309>> livingRenderer = (class_922<class_1309, class_583<class_1309>>) render;
        class_583<class_1309> entityModel = livingRenderer.method_4038();

        if (entityModel instanceof class_572) {

          for (class_572<class_1309> model : models) {
            class_572<class_1309> bipedModel = (class_572<class_1309>) entityModel;
            bipedModel.method_2818(model);
          }
        }
      }
    }
  }
}
