/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.client.render;

import nerdhub.cardinal.components.api.event.ItemComponentCallbackV2;
import net.minecraft.class_1309;
import net.minecraft.class_2960;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;
import net.minecraft.class_4608;
import net.minecraft.class_918;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.CuriosComponent;
import top.theillusivec4.curios.api.type.component.IRenderableCurio;
import top.theillusivec4.curios.client.render.model.AmuletModel;
import top.theillusivec4.curios.client.render.model.CrownModel;
import top.theillusivec4.curios.client.render.model.KnucklesModel;
import top.theillusivec4.curios.common.CuriosRegistry;

public class CuriosRenderComponents {

  private static final class_2960 AMULET_TEXTURE = new class_2960(CuriosApi.MODID,
      "textures/entity/amulet.png");
  private static final class_2960 CROWN_TEXTURE = new class_2960(CuriosApi.MODID,
      "textures/entity/crown.png");
  private static final class_2960 KNUCKLES_TEXTURE = new class_2960(CuriosApi.MODID,
      "textures/entity/knuckles.png");

  public static void register() {
    ItemComponentCallbackV2.event(CuriosRegistry.AMULET).register(
        ((item, itemStack, componentContainer) -> componentContainer
            .put(CuriosComponent.ITEM_RENDER, new IRenderableCurio() {
              AmuletModel<class_1309> model = new AmuletModel<>();

              @Override
              public void render(String identifier, int index, class_4587 matrixStack,
                  class_4597 vertexConsumerProvider, int light,
                  class_1309 livingEntity, float limbSwing, float limbSwingAmount,
                  float partialTicks, float ageInTicks, float netHeadYaw, float headPitch) {
                IRenderableCurio.RenderHelper.translateIfSneaking(matrixStack, livingEntity);
                IRenderableCurio.RenderHelper.rotateIfSneaking(matrixStack, livingEntity);
                class_4588 consumer = class_918
                    .method_23181(vertexConsumerProvider, model.method_23500(AMULET_TEXTURE),
                        false, itemStack.method_7958());
                model.method_2828(matrixStack, consumer, light, class_4608.field_21444, 1.0F, 1.0F,
                    1.0F, 1.0F);
              }
            })));

    ItemComponentCallbackV2.event(CuriosRegistry.CROWN).register(
        ((item, itemStack, componentContainer) -> componentContainer
            .put(CuriosComponent.ITEM_RENDER, new IRenderableCurio() {
              CrownModel<class_1309> model = new CrownModel<>();

              @Override
              public void render(String identifier, int index, class_4587 matrixStack,
                  class_4597 vertexConsumerProvider, int light,
                  class_1309 livingEntity, float limbSwing, float limbSwingAmount,
                  float partialTicks, float ageInTicks, float netHeadYaw, float headPitch) {
                IRenderableCurio.RenderHelper.followHeadRotations(livingEntity, model.crown);
                class_4588 consumer = class_918
                    .method_23181(vertexConsumerProvider, model.method_23500(CROWN_TEXTURE),
                        false, itemStack.method_7958());
                model.method_2828(matrixStack, consumer, light, class_4608.field_21444, 1.0F, 1.0F,
                    1.0F, 1.0F);
              }
            })));

    ItemComponentCallbackV2.event(CuriosRegistry.KNUCKLES).register(
        ((item, itemStack, componentContainer) -> componentContainer
            .put(CuriosComponent.ITEM_RENDER, new IRenderableCurio() {
              KnucklesModel model = new KnucklesModel();

              @Override
              public void render(String identifier, int index, class_4587 matrixStack,
                  class_4597 vertexConsumerProvider, int light,
                  class_1309 livingEntity, float limbSwing, float limbSwingAmount,
                  float partialTicks, float ageInTicks, float netHeadYaw, float headPitch) {
                IRenderableCurio.RenderHelper.followBodyRotations(livingEntity, model);
                model.method_17087(livingEntity, limbSwing, limbSwingAmount, ageInTicks, netHeadYaw,
                    headPitch);
                class_4588 consumer = class_918
                    .method_23181(vertexConsumerProvider,
                        model.method_23500(KNUCKLES_TEXTURE), false, itemStack.method_7958());
                model.method_2828(matrixStack, consumer, light, class_4608.field_21444, 1.0F, 1.0F,
                    1.0F, 1.0F);
              }
            })));
  }
}
