/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.common;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Multimap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.Set;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import net.minecraft.class_1309;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_2378;
import net.minecraft.class_2487;
import net.minecraft.class_2499;
import net.minecraft.class_2960;
import net.minecraft.class_3489;
import net.minecraft.class_3494;
import org.apache.commons.lang3.tuple.ImmutableTriple;
import org.apache.logging.log4j.util.TriConsumer;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.CuriosComponent;
import top.theillusivec4.curios.api.type.component.ICurio;
import top.theillusivec4.curios.api.type.component.ICuriosItemHandler;
import top.theillusivec4.curios.api.type.component.IRenderableCurio;
import top.theillusivec4.curios.api.type.inventory.ICurioStacksHandler;
import top.theillusivec4.curios.api.type.inventory.IDynamicStackHandler;
import top.theillusivec4.curios.api.type.util.ICuriosHelper;

public class CuriosHelper implements ICuriosHelper {

  private static TriConsumer<String, Integer, class_1309> brokenCurioConsumer;

  @Override
  public Optional<ICurio> getCurio(class_1799 stack) {
    return CuriosComponent.ITEM.maybeGet(stack);
  }

  @Override
  public Optional<IRenderableCurio> getRenderableCurio(class_1799 stack) {
    return CuriosComponent.ITEM_RENDER.maybeGet(stack);
  }

  @Override
  public Optional<ICuriosItemHandler> getCuriosHandler(final class_1309 livingEntity) {
    return CuriosComponent.INVENTORY.maybeGet(livingEntity);
  }

  @Override
  public Set<String> getCurioTags(class_1792 item) {
    List<class_2960> list = Lists.newArrayList();

    for (Entry<class_2960, class_3494<class_1792>> identifierTagEntry : class_3489.method_15106().method_15196()
        .entrySet()) {
      if ((identifierTagEntry.getValue()).method_15141(item)) {
        list.add(identifierTagEntry.getKey());
      }
    }
    return list.stream().filter(tag -> tag.method_12836().equals(CuriosApi.MODID))
        .map(class_2960::method_12832).collect(Collectors.toSet());
  }

  @Override
  public Optional<ImmutableTriple<String, Integer, class_1799>> findEquippedCurio(class_1792 item,
      final class_1309 livingEntity) {
    return findEquippedCurio((stack) -> stack.method_7909() == item, livingEntity);
  }

  @Override
  public Optional<ImmutableTriple<String, Integer, class_1799>> findEquippedCurio(
      Predicate<class_1799> filter, final class_1309 livingEntity) {

    ImmutableTriple<String, Integer, class_1799> result = getCuriosHandler(livingEntity)
        .map(handler -> {
          Map<String, ICurioStacksHandler> curios = handler.getCurios();

          for (String id : curios.keySet()) {
            ICurioStacksHandler stacksHandler = curios.get(id);
            IDynamicStackHandler stackHandler = stacksHandler.getStacks();

            for (int i = 0; i < stackHandler.method_5439(); i++) {
              class_1799 stack = stackHandler.method_5438(i);

              if (!stack.method_7960() && filter.test(stack)) {
                return new ImmutableTriple<>(id, i, stack);
              }
            }
          }
          return new ImmutableTriple<>("", 0, class_1799.field_8037);
        }).orElse(new ImmutableTriple<>("", 0, class_1799.field_8037));

    return result.getLeft().isEmpty() ? Optional.empty() : Optional.of(result);
  }

  @Override
  public Multimap<class_1320, class_1322> getAttributeModifiers(String identifier,
      class_1799 stack) {
    Multimap<class_1320, class_1322> multimap;

    if (stack.method_7969() != null && stack.method_7969().method_10573("CurioAttributeModifiers", 9)) {
      multimap = HashMultimap.create();
      class_2499 listnbt = stack.method_7969().method_10554("CurioAttributeModifiers", 10);

      for (int i = 0; i < listnbt.size(); ++i) {
        class_2487 compoundnbt = listnbt.method_10602(i);

        if (!compoundnbt.method_10573("Slot", 8) || compoundnbt.method_10558("Slot").equals(identifier)) {
          class_1320 attribute = class_2378.field_23781
              .method_10223(class_2960.method_12829(compoundnbt.method_10558("AttributeName")));

          if (attribute != null) {
            class_1322 attributemodifier = class_1322
                .method_26859(compoundnbt);

            if (attributemodifier != null
                && attributemodifier.method_6189().getLeastSignificantBits() != 0L
                && attributemodifier.method_6189().getMostSignificantBits() != 0L) {
              multimap.put(attribute, attributemodifier);
            }
          }
        }
      }
      return multimap;
    }
    return getCurio(stack).map(curio -> curio.getAttributeModifiers(identifier))
        .orElse(HashMultimap.create());
  }

  @Override
  public void onBrokenCurio(String id, int index, class_1309 damager) {
    brokenCurioConsumer.accept(id, index, damager);
  }

  @Override
  public void setBrokenCurioConsumer(TriConsumer<String, Integer, class_1309> consumer) {

    if (brokenCurioConsumer == null) {
      brokenCurioConsumer = consumer;
    }
  }
}