/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.common;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.SortedMap;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1542;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2371;
import net.minecraft.class_2487;
import net.minecraft.class_2499;
import net.minecraft.class_2519;
import net.minecraft.class_2540;
import net.minecraft.class_3222;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.type.ISlotType;
import top.theillusivec4.curios.api.type.component.ICuriosItemHandler;
import top.theillusivec4.curios.api.type.inventory.ICurioStacksHandler;
import top.theillusivec4.curios.api.type.inventory.IDynamicStackHandler;
import top.theillusivec4.curios.common.inventory.CurioStacksHandler;

public class PlayerCuriosComponent implements ICuriosItemHandler {

  Map<String, ICurioStacksHandler> curios = new LinkedHashMap<>();
  Set<String> locked = new HashSet<>();
  class_2371<class_1799> invalidStacks = class_2371.method_10211();
  class_1657 wearer;

  public PlayerCuriosComponent(class_1657 playerEntity) {
    this.wearer = playerEntity;
    this.reset();
  }

  @Override
  public Map<String, ICurioStacksHandler> getCurios() {
    return Collections.unmodifiableMap(this.curios);
  }

  @Override
  public void setCurios(Map<String, ICurioStacksHandler> map) {
    this.curios = map;
  }

  @Override
  public int getSlots() {
    return this.curios.values().stream().mapToInt(ICurioStacksHandler::getSlots).sum();
  }

  @Override
  public Set<String> getLockedSlots() {
    return Collections.unmodifiableSet(this.locked);
  }

  @Override
  public void reset() {

    if (!this.wearer.method_5770().method_8608() && wearer instanceof class_3222) {
      this.locked.clear();
      this.curios.clear();
      this.invalidStacks.clear();
      CuriosApi.getSlotHelper().createSlots().forEach(
          ((slotType, stacksHandler) -> curios.put(slotType.getIdentifier(), stacksHandler)));
    }
  }

  @Override
  public Optional<ICurioStacksHandler> getStacksHandler(String identifier) {
    return Optional.ofNullable(this.curios.get(identifier));
  }

  @Override
  public void unlockSlotType(String identifier, int amount, boolean visible, boolean cosmetic) {
    this.curios.putIfAbsent(identifier, new CurioStacksHandler(amount, 0, visible, cosmetic));
    this.locked.remove(identifier);
  }

  @Override
  public void lockSlotType(String identifier) {
    this.getStacksHandler(identifier).ifPresent(stackHandler -> {
      this.curios.remove(identifier);
      this.locked.add(identifier);
      this.loseStacks(stackHandler.getStacks(), identifier, stackHandler.getSlots());
    });
  }

  @Override
  public void growSlotType(String identifier, int amount) {

    if (amount > 0) {
      this.getStacksHandler(identifier).ifPresent(stackHandler -> stackHandler.grow(amount));
    }
  }

  @Override
  public void shrinkSlotType(String identifier, int amount) {
    if (amount > 0) {
      this.getStacksHandler(identifier).ifPresent(stackHandler -> {
        int toShrink = Math.min(stackHandler.getSlots() - 1, amount);
        this.loseStacks(stackHandler.getStacks(), identifier, toShrink);
        stackHandler.shrink(amount);
      });
    }
  }

  @Override
  public class_1309 getWearer() {
    return this.wearer;
  }

  @Override
  public void loseInvalidStack(class_1799 stack) {
    this.invalidStacks.add(stack);
  }

  @Override
  public void handleInvalidStacks() {

    if (this.wearer != null && !this.invalidStacks.isEmpty()) {
      this.invalidStacks.forEach(drop -> giveItemToPlayer(wearer, drop));
      this.invalidStacks = class_2371.method_10211();
    }
  }

  @Override
  public class_1297 getEntity() {
    return this.wearer;
  }

  @Override
  public void fromTag(class_2487 compoundTag) {
    class_2499 tagList = compoundTag.method_10554("Curios", 10);
    class_2499 lockedList = compoundTag.method_10554("Locked", 8);

    if (!tagList.isEmpty()) {
      Map<String, ICurioStacksHandler> curios = new LinkedHashMap<>();
      SortedMap<ISlotType, ICurioStacksHandler> sortedCurios = CuriosApi.getSlotHelper()
          .createSlots();

      for (int i = 0; i < tagList.size(); i++) {
        class_2487 tag = tagList.method_10602(i);
        String identifier = tag.method_10558("Identifier");
        CurioStacksHandler prevStacksHandler = new CurioStacksHandler();
        prevStacksHandler.deserializeTag(tag.method_10562("StacksHandler"));

        Optional<ISlotType> optionalType = CuriosApi.getSlotHelper().getSlotType(identifier);
        optionalType.ifPresent(type -> {
          CurioStacksHandler newStacksHandler = new CurioStacksHandler(type.getSize(),
              prevStacksHandler.getSizeShift(), type.isVisible(), type.hasCosmetic());
          int index = 0;

          while (index < newStacksHandler.getSlots() && index < prevStacksHandler.getSlots()) {
            newStacksHandler.getStacks()
                .method_5447(index, prevStacksHandler.getStacks().method_5438(index));
            newStacksHandler.getCosmeticStacks()
                .method_5447(index, prevStacksHandler.getCosmeticStacks().method_5438(index));
            index++;
          }

          while (index < prevStacksHandler.getSlots()) {
            this.loseInvalidStack(prevStacksHandler.getStacks().method_5438(index));
            this.loseInvalidStack(prevStacksHandler.getCosmeticStacks().method_5438(index));
            index++;
          }
          sortedCurios.put(type, newStacksHandler);

          for (int j = 0;
              j < newStacksHandler.getRenders().size() && j < prevStacksHandler.getRenders().size();
              j++) {
            newStacksHandler.getRenders().set(j, prevStacksHandler.getRenders().get(j));
          }
        });

        if (!optionalType.isPresent()) {
          IDynamicStackHandler stackHandler = prevStacksHandler.getStacks();
          IDynamicStackHandler cosmeticStackHandler = prevStacksHandler.getCosmeticStacks();

          for (int j = 0; j < stackHandler.method_5439(); j++) {
            class_1799 stack = stackHandler.method_5438(j);

            if (!stack.method_7960()) {
              this.loseInvalidStack(stack);
            }

            class_1799 cosmeticStack = cosmeticStackHandler.method_5438(j);

            if (!cosmeticStack.method_7960()) {
              this.loseInvalidStack(cosmeticStack);
            }
          }
        }
      }
      sortedCurios.forEach(
          (slotType, stacksHandler) -> curios.put(slotType.getIdentifier(), stacksHandler));
      this.setCurios(curios);

      for (int k = 0; k < lockedList.size(); k++) {
        this.lockSlotType(lockedList.method_10608(k));
      }
    }
  }

  @Override
  public class_2487 toTag(class_2487 compoundTag) {

    class_2499 taglist = new class_2499();
    this.getCurios().forEach((key, stacksHandler) -> {
      class_2487 tag = new class_2487();
      tag.method_10566("StacksHandler", stacksHandler.serializeTag());
      tag.method_10582("Identifier", key);
      taglist.add(tag);
    });
    compoundTag.method_10566("Curios", taglist);

    class_2499 taglist1 = new class_2499();

    for (String identifier : this.getLockedSlots()) {
      taglist1.add(class_2519.method_23256(identifier));
    }
    compoundTag.method_10566("Locked", taglist1);
    return compoundTag;
  }

  @Override
  public void writeToPacket(class_2540 buf) {
    buf.writeInt(this.curios.size());

    for (Map.Entry<String, ICurioStacksHandler> entry : this.curios.entrySet()) {
      buf.method_10814(entry.getKey());
      buf.method_10794(entry.getValue().serializeTag());
    }
  }

  @Override
  public void readFromPacket(class_2540 buf) {
    int entrySize = buf.readInt();
    Map<String, ICurioStacksHandler> map = new LinkedHashMap<>();

    for (int i = 0; i < entrySize; i++) {
      String key = buf.method_10800(25);
      CurioStacksHandler stacksHandler = new CurioStacksHandler();
      class_2487 compound = buf.method_10798();

      if (compound != null) {
        stacksHandler.deserializeTag(compound);
      }
      map.put(key, stacksHandler);
    }
    this.setCurios(map);
  }

  private void loseStacks(IDynamicStackHandler stackHandler, String identifier, int amount) {

    if (this.wearer != null && !this.wearer.method_5770().method_8608()) {
      List<class_1799> drops = new ArrayList<>();

      for (int i = stackHandler.method_5439() - amount; i < stackHandler.method_5439(); i++) {
        class_1799 stack = stackHandler.method_5438(i);
        drops.add(stackHandler.method_5438(i));

        if (!stack.method_7960()) {
          wearer.method_6127().method_26847(
              CuriosApi.getCuriosHelper().getAttributeModifiers(identifier, stack));
          int index = i;
          CuriosApi.getCuriosHelper().getCurio(stack)
              .ifPresent(curio -> curio.onUnequip(identifier, index, this.wearer));
        }
        stackHandler.method_5447(i, class_1799.field_8037);
      }
      drops.forEach(drop -> giveItemToPlayer(wearer, drop));
    }
  }

  private static void giveItemToPlayer(class_1657 playerEntity, class_1799 stack) {
    boolean bl = playerEntity.field_7514.method_7394(stack);
    class_1542 itemEntity;

    if (bl && stack.method_7960()) {
      stack.method_7939(1);
      itemEntity = playerEntity.method_7328(stack, false);

      if (itemEntity != null) {
        itemEntity.method_6987();
      }
      playerEntity.field_6002
          .method_8465(null, playerEntity.method_23317(), playerEntity.method_23318(), playerEntity.method_23321(),
              class_3417.field_15197, class_3419.field_15248, 0.2F,
              ((playerEntity.method_6051().nextFloat() - playerEntity.method_6051().nextFloat()) * 0.7F
                  + 1.0F) * 2.0F);
      playerEntity.field_7498.method_7623();
    } else {
      itemEntity = playerEntity.method_7328(stack, false);

      if (itemEntity != null) {
        itemEntity.method_6975();
        itemEntity.method_6984(playerEntity.method_5667());
      }
    }
  }
}
